/*
	file automatically generated by make_test_files.pl
	Fri Jan 17 09:21:12 2003
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for TDR
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_tdr_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


/* prototypes */
double pdf( double x, const UNUR_DISTR *distr );

double pdf_bimodal( double x, const UNUR_DISTR *distr );
double dpdf_bimodal( double x, const UNUR_DISTR *distr );

double pdf_negative( double x, const UNUR_DISTR *distr );
double dpdf_negative( double x, const UNUR_DISTR *distr );

double pdf_partnegative( double x, const UNUR_DISTR *distr );
double dpdf_partnegative( double x, const UNUR_DISTR *distr );

double pdf_sqrtlin( double x, const UNUR_DISTR *distr );
double dpdf_sqrtlin( double x, const UNUR_DISTR *distr );
double cdf_sqrtlin( double x, const UNUR_DISTR *distr );

double pdf_sqrtlinshft(double x, const UNUR_DISTR *distr );
double dpdf_sqrtlinshft( double x, const UNUR_DISTR *distr );
double cdf_sqrtlinshft( double x, const UNUR_DISTR *distr );

#define COMPARE_SAMPLE_SIZE   (10000)
#define VIOLATE_SAMPLE_SIZE   (20)





/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_tdr_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_tdr_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

        /* seed build-in uniform generators */
        unur_urng_MRG31k3p_seed(52232);
	unur_urng_fish_seed(52232);
	unur_urng_mstd_seed(52232);

	/* seed uniform random number generator */
#if UNUR_URNG_TYPE == UNUR_URNG_PRNG
	unur_set_default_urng(prng_new("mt19937(52232)"));
#elif UNUR_URNG_TYPE == UNUR_URNG_RNGSTREAM
	{
	    unsigned long seed[6] = {52232,52232+1,52232+2,52232+3,52232+4,52232+5};
	    RngStream_SetPackageSeed(seed);
	}
#endif  /* UNUR_URNG_TYPE */

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("tdr: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* free memory */
        compare_free_memory();

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 45, (unur_tdr_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 45, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 51, (unur_tdr_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 51, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* data missing in distribution object */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
/* pdf, dpdf */
n_tests_failed += check_expected_NULL( TESTLOG, 58, (unur_tdr_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 58, UNUR_ERR_DISTR_REQUIRED );

unur_errno = 0;
/* dpdf */
unur_distr_cont_set_pdf(distr,pdf);
n_tests_failed += check_expected_NULL( TESTLOG, 63, (unur_tdr_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 63, UNUR_ERR_DISTR_REQUIRED );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_PAR   *par = NULL;
   par = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 73, (unur_tdr_set_cpoints( par, 0, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 73, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 76, (unur_tdr_set_guidefactor( par, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 76, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 79, (unur_tdr_set_max_sqhratio( par, 0.95 )) );
n_tests_failed += check_errorcode( TESTLOG, 79, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 82, (unur_tdr_set_max_intervals( par, 100 )) );
n_tests_failed += check_errorcode( TESTLOG, 82, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 85, (unur_tdr_set_center( par, 0. )) );
n_tests_failed += check_errorcode( TESTLOG, 85, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 88, (unur_tdr_set_usecenter( par, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 88, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 91, (unur_tdr_set_usemode( par, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 91, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 94, (unur_tdr_set_variant_gw( par )) );
n_tests_failed += check_errorcode( TESTLOG, 94, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 97, (unur_tdr_set_variant_ps( par )) );
n_tests_failed += check_errorcode( TESTLOG, 97, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 100, (unur_tdr_set_variant_ia( par )) );
n_tests_failed += check_errorcode( TESTLOG, 100, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 103, (unur_tdr_set_usedars( par, TRUE )) );
n_tests_failed += check_errorcode( TESTLOG, 103, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 106, (unur_tdr_set_darsfactor( par, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 106, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 109, (unur_tdr_set_c( par, -0.5 )) );
n_tests_failed += check_errorcode( TESTLOG, 109, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 112, (unur_tdr_set_verify( par, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 112, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 115, (unur_tdr_set_pedantic( par, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 115, UNUR_ERR_NULL );
}

/* invalid parameter object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = unur_arou_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 124, (unur_tdr_set_cpoints( par, 0, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 124, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 127, (unur_tdr_set_guidefactor( par, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 127, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 130, (unur_tdr_set_max_sqhratio( par, 0.95 )) );
n_tests_failed += check_errorcode( TESTLOG, 130, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 133, (unur_tdr_set_max_intervals( par, 100 )) );
n_tests_failed += check_errorcode( TESTLOG, 133, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 136, (unur_tdr_set_center( par, 0. )) );
n_tests_failed += check_errorcode( TESTLOG, 136, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 139, (unur_tdr_set_usecenter( par, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 139, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 142, (unur_tdr_set_usemode( par, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 142, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 145, (unur_tdr_set_variant_gw( par )) );
n_tests_failed += check_errorcode( TESTLOG, 145, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 148, (unur_tdr_set_variant_ps( par )) );
n_tests_failed += check_errorcode( TESTLOG, 148, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 151, (unur_tdr_set_variant_ia( par )) );
n_tests_failed += check_errorcode( TESTLOG, 151, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 154, (unur_tdr_set_usedars( par, TRUE )) );
n_tests_failed += check_errorcode( TESTLOG, 154, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 157, (unur_tdr_set_darsfactor( par, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 157, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 160, (unur_tdr_set_c( par, -0.5 )) );
n_tests_failed += check_errorcode( TESTLOG, 160, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 163, (unur_tdr_set_verify( par, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 163, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 166, (unur_tdr_set_pedantic( par, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 166, UNUR_ERR_PAR_INVALID );
free(par);
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double stp[] = {1.,0.,1.};
   int n_stp = 3;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = unur_tdr_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 178, (unur_tdr_set_cpoints( par, -1, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 178, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 181, (unur_tdr_set_cpoints( par, n_stp, stp )) );
n_tests_failed += check_errorcode( TESTLOG, 181, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 184, (unur_tdr_set_guidefactor( par, -1. )) );
n_tests_failed += check_errorcode( TESTLOG, 184, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 187, (unur_tdr_set_max_sqhratio( par, -1. )) );
n_tests_failed += check_errorcode( TESTLOG, 187, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 190, (unur_tdr_set_max_sqhratio( par, 2. )) );
n_tests_failed += check_errorcode( TESTLOG, 190, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 193, (unur_tdr_set_max_intervals( par, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 193, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 196, (unur_tdr_set_usedars( par, -1 )) );
n_tests_failed += check_errorcode( TESTLOG, 196, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 199, (unur_tdr_set_darsfactor( par, -1. )) );
n_tests_failed += check_errorcode( TESTLOG, 199, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 202, (unur_tdr_set_c( par, 0.5 )) );
n_tests_failed += check_errorcode( TESTLOG, 202, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 205, (unur_tdr_set_c( par, -0.6 )) );
n_tests_failed += check_errorcode( TESTLOG, 205, UNUR_ERR_PAR_SET );

unur_errno = 0;
unur_tdr_set_c( par, -0.3 );
n_tests_failed += check_errorcode( TESTLOG, 208, UNUR_ERR_PAR_SET );
free(par);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid generator object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = unur_arou_new(distr);
   gen = unur_init(par); 
abort_if_NULL(TESTLOG, 219,    gen );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 222, (unur_tdr_get_sqhratio(gen)) );
n_tests_failed += check_errorcode( TESTLOG, 222, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 225, (unur_tdr_get_hatarea(gen)) );
n_tests_failed += check_errorcode( TESTLOG, 225, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 228, (unur_tdr_get_squeezearea(gen)) );
n_tests_failed += check_errorcode( TESTLOG, 228, UNUR_ERR_GEN_INVALID );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid generator object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = unur_arou_new(distr);
   unur_set_debug(par,0);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 240,    gen );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 243, (unur_tdr_chg_verify(gen,1)) );
n_tests_failed += check_errorcode( TESTLOG, 243, UNUR_ERR_GEN_INVALID );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_PAR   *par = NULL;
   par = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 254, (unur_init( par )) );
n_tests_failed += check_errorcode( TESTLOG, 254, UNUR_ERR_NULL );
}

/* stp out of domain */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double stp[] = {-2.5,-1.,0.,1.,2.5};
   int n_stp = 5;
   gen = NULL;
   distr = unur_distr_normal(NULL,0);
   unur_distr_cont_set_domain(distr,-2.,2.);
   par = unur_tdr_new(distr);
   unur_tdr_set_usedars(par,FALSE);
   unur_tdr_set_cpoints( par, n_stp, stp ); 

unur_errno = 0;
gen = unur_init( par );
n_tests_failed += check_errorcode( TESTLOG, 267, UNUR_ERR_GEN_DATA );
unur_free(gen);
unur_distr_free(distr);
}

/* bimodal */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   gen = NULL;
   distr = unur_distr_cont_new(); 
   unur_distr_cont_set_pdf(distr,pdf_bimodal);
   unur_distr_cont_set_dpdf(distr,dpdf_bimodal);
   par = unur_tdr_new(distr);
   unur_tdr_set_usedars(par,FALSE);
   unur_tdr_set_cpoints( par, 30, NULL ); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 279, (gen = unur_init( par )) );
n_tests_failed += check_errorcode( TESTLOG, 279, UNUR_ERR_GEN_CONDITION );
unur_free(gen);
unur_distr_free(distr);
}

/* wrong mode */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = NULL; 

unur_errno = 0;
unur_distr_cont_set_mode(distr,-1.);
par = unur_tdr_new(distr);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints( par, 30, NULL );
n_tests_failed += check_expected_NULL( TESTLOG, 291, (gen = unur_init( par )) );
n_tests_failed += check_errorcode( TESTLOG, 291, UNUR_ERR_GEN_DATA );

unur_errno = 0;
unur_distr_cont_set_mode(distr,1.);
par = unur_tdr_new(distr);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints( par, 30, NULL );
gen = unur_init( par );
n_tests_failed += check_errorcode( TESTLOG, 298, UNUR_ERR_GEN_DATA );
unur_free(gen);
unur_distr_free(distr);
}

/* negative value of pdf */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   gen = NULL;
   distr = unur_distr_cont_new(); 
   unur_distr_cont_set_pdf(distr,pdf_negative);
   unur_distr_cont_set_dpdf(distr,dpdf_negative);
   par = NULL; 

unur_errno = 0;
par = unur_tdr_new(distr);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints( par, 30, NULL );
n_tests_failed += check_expected_NULL( TESTLOG, 311, (gen = unur_init( par )) );
n_tests_failed += check_errorcode( TESTLOG, 311, UNUR_ERR_GEN_DATA );

unur_errno = 0;
unur_distr_cont_set_domain(distr,-2.,2.);
par = unur_tdr_new(distr);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints( par, 30, NULL );
n_tests_failed += check_expected_NULL( TESTLOG, 318, (gen = unur_init( par )) );
n_tests_failed += check_errorcode( TESTLOG, 318, UNUR_ERR_GEN_DATA );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_GEN   *gen = NULL;
   gen = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 332, (unur_tdr_eval_invcdfhat(gen,0.5,NULL)) );
n_tests_failed += check_errorcode( TESTLOG, 332, UNUR_ERR_NULL );
unur_free(gen);
}

/* invalid generator object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = unur_arou_new(distr);
   gen = unur_init(par); 
abort_if_NULL(TESTLOG, 339,    gen );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 342, (unur_tdr_eval_invcdfhat(gen,0.5,NULL)) );
n_tests_failed += check_errorcode( TESTLOG, 342, UNUR_ERR_GEN_INVALID );
unur_free(gen);
unur_distr_free(distr);
}

/* invalid domain */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = unur_tdr_new(distr);
   unur_set_debug(par,0);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 350,    gen );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 353, (unur_tdr_eval_invcdfhat(gen,1.5,NULL)) );
n_tests_failed += check_errorcode( TESTLOG, 353, UNUR_ERR_DOMAIN );

unur_errno = 0;
n_tests_failed += check_expected_negINFINITY( TESTLOG, 356, (unur_tdr_eval_invcdfhat(gen,-0.5,NULL)) );
n_tests_failed += check_errorcode( TESTLOG, 356, UNUR_ERR_DOMAIN );
unur_free(gen);
unur_distr_free(distr);
}

/* partial negative value of pdf */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   int i;
   double x;
   double stp[] = {-0.9,0.9};
   int n_stp = 2;
   distr = unur_distr_cont_new(); 
   unur_distr_cont_set_pdf(distr,pdf_partnegative);
   unur_distr_cont_set_dpdf(distr,dpdf_partnegative);
   par = unur_tdr_new(distr);
   unur_tdr_set_usedars(par,FALSE);
   unur_tdr_set_cpoints( par, n_stp, stp );
   unur_tdr_set_pedantic( par, 1 );
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 372,    gen );

unur_errno = 0;
for (i=0; i<100; i++) { x = unur_sample_cont(gen); }
n_tests_failed += check_expected_INFINITY( TESTLOG, 376, (x) );
n_tests_failed += check_errorcode( TESTLOG, 376, UNUR_ERR_GEN_CONDITION );
unur_free(gen);
unur_distr_free(distr);
}

/* compare */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   distr = unur_distr_normal(NULL,0); 
   par = NULL; 

unur_errno = 0;
/* GW, c = -0.5 */
par = unur_tdr_new(distr);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += compare_double_sequence_par_start( TESTLOG, 391, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* GW, c = -0.5 - verifying mode */
par = unur_tdr_new(distr);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_verify(par,1);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += compare_double_sequence_par( TESTLOG, 399, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* PS, c = -0.5 */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += compare_double_sequence_par_start( TESTLOG, 406, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* PS, c = -0.5 - verifying mode */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_verify(par,1);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += compare_double_sequence_par( TESTLOG, 414, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* IA, c = -0.5 */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += compare_double_sequence_par_start( TESTLOG, 421, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* IA, c = -0.5 - verifying mode */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_verify(par,1);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += compare_double_sequence_par( TESTLOG, 429, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* GW, c = 0 */
par = unur_tdr_new(distr);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += compare_double_sequence_par_start( TESTLOG, 439, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* GW, c = 0, verifying mode */
par = unur_tdr_new(distr);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0);
unur_tdr_set_verify(par,1);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += compare_double_sequence_par( TESTLOG, 447, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* PS, c = 0 */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += compare_double_sequence_par_start( TESTLOG, 454, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* PS, c = 0, verifying mode */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0);
unur_tdr_set_verify(par,1);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += compare_double_sequence_par( TESTLOG, 462, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* IA, c = 0 */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += compare_double_sequence_par_start( TESTLOG, 469, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* IA, c = 0, verifying mode */
par = unur_tdr_new(distr);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0);
unur_tdr_set_verify(par,1);
unur_tdr_set_usedars(par,FALSE);
n_tests_failed += compare_double_sequence_par( TESTLOG, 477, par, COMPARE_SAMPLE_SIZE );
unur_distr_free(distr);
}

/* compare clone */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   UNUR_GEN *clone;
   distr = unur_distr_normal(NULL,0); 
   par = NULL;
   gen = NULL; 

unur_errno = 0;
/* original generator object */
par = unur_tdr_new(distr);
unur_tabl_set_max_sqhratio(par,0.);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 491, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* clone */
clone = unur_gen_clone(gen);
unur_free(gen);
gen = clone;
n_tests_failed += compare_double_sequence_gen( TESTLOG, 497, gen, COMPARE_SAMPLE_SIZE );
unur_free(gen);
unur_distr_free(distr);
}

/* compare stringparser */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {3.,3.};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_center(par,2.1);
unur_tdr_set_cpoints(par,20,NULL);
unur_tdr_set_guidefactor(par,2.);
unur_tdr_set_max_intervals(par,32);
unur_tdr_set_max_sqhratio(par,0.9);
unur_tdr_set_usecenter(par,TRUE);
unur_tdr_set_verify(par,TRUE);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 517, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; center = 2.1; usecenter; cpoints = 20; guidefactor = 2.; \
  max_intervals = 32; max_sqhratio = 0.9; verify" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 524, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_cpoints(par,9,NULL);
unur_tdr_set_pedantic(par,TRUE);
unur_tdr_set_usemode(par,FALSE);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 534, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; cpoints = 9; pedantic; usemode = false" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 540, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_c(par,0.);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 548, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; c = 0." );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 554, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_c(par,-0.5);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 562, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; c = -0.5" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 568, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_variant_gw(par);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 576, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; variant_gw" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 582, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_variant_ia(par);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 590, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; variant_ia" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 596, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_variant_ps(par);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 604, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; variant_ps" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 610, gen, COMPARE_SAMPLE_SIZE );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[30];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 30 */
fpm[0] = 1.;
fpm[1] = 2.;
distr[0] = unur_distr_beta(fpm,2);


fpm[0] = 1.;
fpm[1] = 5.;
distr[1] = unur_distr_beta(fpm,2);


fpm[0] = 1.;
fpm[1] = 100.;
distr[2] = unur_distr_beta(fpm,2);


fpm[0] = 3.;
fpm[1] = 4.;
distr[3] = unur_distr_beta(fpm,2);


fpm[0] = 5.;
fpm[1] = 100.;
distr[4] = unur_distr_beta(fpm,2);


fpm[0] = 500.;
fpm[1] = 300.;
distr[5] = unur_distr_beta(fpm,2);


fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = -3.;
fpm[3] = 15.;
distr[6] = unur_distr_beta(fpm,4);


distr[7] = unur_distr_cauchy(NULL,0);


fpm[0] = 1.;
fpm[1] = 20.;
distr[8] = unur_distr_cauchy(fpm,2);


distr[24] = unur_distr_exponential(NULL,0);


fpm[0] = 30.;
fpm[1] = -5.;
distr[25] = unur_distr_exponential(fpm,2);


fpm[0] = 1.;
distr[9] = unur_distr_gamma(fpm,1);


fpm[0] = 2.;
distr[10] = unur_distr_gamma(fpm,1);


fpm[0] = 3.;
distr[11] = unur_distr_gamma(fpm,1);


fpm[0] = 10.;
distr[12] = unur_distr_gamma(fpm,1);


fpm[0] = 1000.;
distr[13] = unur_distr_gamma(fpm,1);


fpm[0] = 5.;
fpm[1] = 1000.;
distr[14] = unur_distr_gamma(fpm,2);


fpm[0] = 5.;
fpm[1] = 1.e-5;
distr[15] = unur_distr_gamma(fpm,2);


fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = 1000;
distr[16] = unur_distr_gamma(fpm,3);


distr[26] = unur_distr_laplace(NULL,0);


fpm[0] = -10.;
fpm[1] = 100.;
distr[27] = unur_distr_laplace(fpm,2);


distr[17] = unur_distr_normal(NULL,0);


fpm[0] = 1.;
fpm[1] = 1.e-5;
distr[18] = unur_distr_normal(fpm,2);


fpm[0] = 0.;
fpm[1] = 1.e+5;
distr[19] = unur_distr_normal(fpm,2);


distr[20] = unur_distr_uniform(NULL,0);


fpm[0] = 1.;
fpm[1] = 20.;
distr[21] = unur_distr_uniform(fpm,2);


distr[28] = unur_distr_cont_new();
unur_distr_cont_set_pdf(distr[28],pdf_sqrtlin);
unur_distr_cont_set_dpdf(distr[28],dpdf_sqrtlin);
unur_distr_cont_set_cdf(distr[28],cdf_sqrtlin);
unur_distr_set_name(distr[28],"sqrtlin");


distr[29] = unur_distr_cont_new();
unur_distr_cont_set_pdf(distr[29],pdf_sqrtlinshft);
unur_distr_cont_set_dpdf(distr[29],dpdf_sqrtlinshft);
unur_distr_cont_set_cdf(distr[29],cdf_sqrtlinshft);
unur_distr_set_name(distr[29],"sqrtlin");


distr[22] = unur_distr_cauchy(NULL,0);
unur_distr_cont_set_domain(distr[22],0.1,1.);
unur_distr_cont_upd_mode(distr[22]);
unur_distr_cont_upd_pdfarea(distr[22]);


fpm[0] = 3.;
fpm[1] = 4.;
distr[23] = unur_distr_beta(fpm,2);
unur_distr_cont_set_domain(distr[23],-2.,5.);


	printf("\n(chi^2) "); fflush(stdout);

/* chi^2 tests: 570 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nChi^2 Test:\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '-' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '-' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '-' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '-' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '-' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '-' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '-' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '-' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [24] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [25] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [10] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [13] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [14] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [15] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [16] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [26] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [27] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [17] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [18] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [19] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [20] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [21] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [22] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [23] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [28] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [29] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '0' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[29]);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("\n(verify hat) "); fflush(stdout);

/* verify hat tests: 570 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nVerify Hat Test (squeeze <= PDF <= hat):\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '0' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '-' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '-' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '0' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '-' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '-' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '0' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '0' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '-' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '-' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '0' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '-' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '-' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '0' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [24] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[24]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [25] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[25]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [10] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [13] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [14] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [15] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [16] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[16]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [26] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[26]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [27] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[27]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [17] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[17]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [18] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[18]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [19] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[19]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [20] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[20]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [21] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[21]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [22] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[22]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [23] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,0.);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,0.);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[23]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [28] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_gw(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) {
unur_tdr_chg_truncated(gen,0.5,0.9);
	}
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ia(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[28]);
	unur_tdr_set_pedantic(par,0);
unur_tdr_set_variant_ps(par);
unur_tdr_set_c(par,-0.5);
unur_tdr_set_usedars(par,FALSE);
unur_tdr_set_cpoints(par,10,NULL);
unur_tdr_set_max_sqhratio(par,0.25);
unur_use_urng_aux_default(par);
	gen = unur_init(par);
	if (gen) unur_tdr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [29] */

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);


/* free distributions */
	unur_distr_free(distr[0]);
	unur_distr_free(distr[1]);
	unur_distr_free(distr[2]);
	unur_distr_free(distr[3]);
	unur_distr_free(distr[4]);
	unur_distr_free(distr[5]);
	unur_distr_free(distr[6]);
	unur_distr_free(distr[7]);
	unur_distr_free(distr[8]);
	unur_distr_free(distr[9]);
	unur_distr_free(distr[10]);
	unur_distr_free(distr[11]);
	unur_distr_free(distr[12]);
	unur_distr_free(distr[13]);
	unur_distr_free(distr[14]);
	unur_distr_free(distr[15]);
	unur_distr_free(distr[16]);
	unur_distr_free(distr[17]);
	unur_distr_free(distr[18]);
	unur_distr_free(distr[19]);
	unur_distr_free(distr[20]);
	unur_distr_free(distr[21]);
	unur_distr_free(distr[22]);
	unur_distr_free(distr[23]);
	unur_distr_free(distr[24]);
	unur_distr_free(distr[25]);
	unur_distr_free(distr[26]);
	unur_distr_free(distr[27]);
	unur_distr_free(distr[28]);
	unur_distr_free(distr[29]);
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	test_ok &= (n_tests_failed>2) ? 0 : 1;   /* we accept 2 failure */
	(n_tests_failed>2) ? printf(" --> failed] ") : printf(" --> ok] ");

	/* prevent compiler from making useless annoying warnings */
	distr[0] = NULL;
	par = NULL;
	gen = NULL;
	darray = NULL;
	fpm[0] = 0.;

} /* end of test_validate */

/*---------------------------------------------------------------------------*/


/* pdf of bimodal density */
double pdf_bimodal( double x, const UNUR_DISTR *distr )
{ 
	return ( exp(-(x-1.)*(x-1.)) + exp(-(x+1.)*(x+1.)) );
}
double dpdf_bimodal( double x, const UNUR_DISTR *distr )
{ 
	return ( -2. * (x-1.) * exp(-(x-1.)*(x-1.)) -2. * (x+1.) * exp(-(x+1.)*(x+1.)) );
}

/* pdf with negative value */
double pdf_negative( double x, const UNUR_DISTR *distr )  { return (-x*x); }
double dpdf_negative( double x, const UNUR_DISTR *distr ) { return (-2.*x); }

/* pdf with partial negative value */
double pdf_partnegative( double x, const UNUR_DISTR *distr )  
{ 
	return ((x>-0.89 && x<0.89) ? -1.: exp(-x*x)); 
}
double dpdf_partnegative( double x, const UNUR_DISTR *distr ) 
{
	return ((x>-0.89 && x<0.89) ?0.: -2.*x*exp(-x*x));
}

/* pdf of normal density */
double pdf( double x, const UNUR_DISTR *distr )
{ 
	return exp(-x*x/2.);
} /* end of pdf */

/* pdf with piecewise linear function as transformed density with T = -1/sqrt */
double pdf_sqrtlin( double x, const UNUR_DISTR *distr )
{ 
	double y = 1./(fabs(x)+1.);
	return y*y;
}
double dpdf_sqrtlin( double x, const UNUR_DISTR *distr )
{ 
	double y = 1./(fabs(x)+1.);
	y = 2.*y*y*y;
	return ((x<0.) ? y : - y);
}
double cdf_sqrtlin( double x, const UNUR_DISTR *distr )
{ 
	if (x<=0.)
		return 0.5/(1.-x);
	else
		return (1.-0.5/(1.+x));
}

/* pdf with piecewise linear function as transformed density with T = -1/sqrt and shifted mode */
double pdf_sqrtlinshft( double x, const UNUR_DISTR *distr )
{ 
	double y;
	x -= 1000.;
	y = 1./(fabs(x)+1.);
	return y*y;
}
double dpdf_sqrtlinshft( double x, const UNUR_DISTR *distr )
{ 
	double y;
	x -= 1000.;
	y = 1./(fabs(x)+1.);
	y = 2.*y*y*y;
	return ((x<0.) ? y : - y);
}
double cdf_sqrtlinshft( double x, const UNUR_DISTR *distr )
{ 
	x -= 1000.;
	if (x<=0.)
		return 0.5/(1.-x);
	else
		return (1.-0.5/(1.+x));
}




/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_tdr_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

